@@ -10,7 +10,7 @@ use crate::arch;
 use crate::synch::spinlock::SpinlockIrqSave;
 use core::fmt;
 
-pub struct Console;
+pub struct Console(());
 
 /// A collection of methods that are required to format
 /// a message to HermitCore's console.
@@ -31,7 +31,13 @@ impl fmt::Write for Console {
 	}
 }
 
-pub static CONSOLE: SpinlockIrqSave<Console> = SpinlockIrqSave::new(Console);
+impl Console {
+	pub fn write_all(&mut self, buf: &[u8]) {
+		arch::output_message_buf(buf)
+	}
+}
+
+pub static CONSOLE: SpinlockIrqSave<Console> = SpinlockIrqSave::new(Console(()));
 
 #[cfg(not(target_os = "hermit"))]
 #[test]
