@@ -40,7 +40,7 @@ use crate::drivers::virtio::depr::virtio_fs;
 // one MUST adjust the associated From<u16>
 // implementation, in order catch all cases correctly,
 // as this function uses the catch-all "_" case!
-#[allow(dead_code, non_camel_case_types)]
+#[allow(dead_code, non_camel_case_types, clippy::upper_case_acronyms)]
 #[repr(u16)]
 pub enum DevId {
 	INVALID = 0x0,
@@ -96,7 +96,7 @@ impl From<u16> for DevId {
 // one MUST adjust the associated From<u8>
 // implementation, in order catch all cases correctly,
 // as this function uses the catch-all "_" case!
-#[allow(dead_code, non_camel_case_types)]
+#[allow(dead_code, non_camel_case_types, clippy::upper_case_acronyms)]
 #[derive(Copy, Clone, PartialEq)]
 #[repr(u8)]
 pub enum CfgType {
@@ -253,7 +253,7 @@ impl Eq for PciCapRaw {}
 // In order to compare two PciCapRaw structs PartialEq is needed
 impl PartialEq for PciCapRaw {
 	fn eq(&self, other: &Self) -> bool {
-		if self.cap_vndr == other.cap_vndr
+		self.cap_vndr == other.cap_vndr
 			&& self.cap_next == other.cap_next
 			&& self.cap_len == other.cap_len
 			&& self.cfg_type == other.cfg_type
@@ -261,11 +261,6 @@ impl PartialEq for PciCapRaw {
 			&& self.id == other.id
 			&& self.offset == other.offset
 			&& self.length == other.length
-		{
-			true
-		} else {
-			false
-		}
 	}
 }
 
@@ -422,12 +417,11 @@ impl<'a> VqCfgHandler<'a> {
 	pub fn set_vq_size(&mut self, size: u16) -> u16 {
 		self.raw.queue_select = self.vq_index;
 
-		if self.raw.queue_size < size {
-			self.raw.queue_size
-		} else {
+		if self.raw.queue_size >= size {
 			self.raw.queue_size = size;
-			self.raw.queue_size
 		}
+
+		self.raw.queue_size
 	}
 
 	pub fn set_ring_addr(&mut self, addr: PhysAddr) {
@@ -462,7 +456,7 @@ impl ComCfg {
 	/// returns `Some(VqCfgHandler)`.
 	///
 	/// INFO: The queue size is automatically bounded by constant `src::config:VIRTIO_MAX_QUEUE_SIZE`.
-	pub fn select_vq(&mut self, index: u16) -> Option<VqCfgHandler> {
+	pub fn select_vq(&mut self, index: u16) -> Option<VqCfgHandler<'_>> {
 		self.com_cfg.queue_select = index;
 
 		if self.com_cfg.queue_size == 0 {
@@ -667,10 +661,10 @@ impl NotifCfg {
 		// See Virtio specification v1.1. - 4.1.4.4
 		//
 		// Base address here already includes offset!
-		let base_addr = VirtMemAddr::from(cap.bar.mem_addr + cap.offset);
+		let base_addr = cap.bar.mem_addr + cap.offset;
 
 		Some(NotifCfg {
-			base_addr: base_addr,
+			base_addr,
 			notify_off_multiplier,
 			rank: cap.id,
 			length: cap.length,
@@ -720,7 +714,7 @@ impl NotifCtrl {
 		if self.f_notif_data {
 			unsafe {
 				let notif_area = core::slice::from_raw_parts_mut(self.notif_addr as *mut u8, 4);
-				let mut notif_data = notif_data.into_iter();
+				let mut notif_data = notif_data.iter();
 
 				for byte in notif_area {
 					*byte = *notif_data.next().unwrap();
@@ -729,7 +723,7 @@ impl NotifCtrl {
 		} else {
 			unsafe {
 				let notif_area = core::slice::from_raw_parts_mut(self.notif_addr as *mut u8, 2);
-				let mut notif_data = notif_data.into_iter();
+				let mut notif_data = notif_data.iter();
 
 				for byte in notif_area {
 					*byte = *notif_data.next().unwrap();
@@ -1009,15 +1003,12 @@ fn read_cap_raw(adapter: &PciAdapter, register: u32) -> PciCapRaw {
 	debug!("Converting read word from PCI device config space into native endian bytes.");
 
 	// Write words sequentialy into array
-	let mut index = 0;
-	for i in 0..4u32 {
+	for i in 0..4 {
 		// Read word need to be converted to little endian bytes as PCI is little endian.
 		// Intepretation of multi byte values needs to be swapped for big endian machines
 		let word: [u8; 4] = env::pci::read_config(adapter, register + 4 * i).to_le_bytes();
-		for j in 0..4 {
-			quadruple_word[index] = word[j];
-			index += 1;
-		}
+		let i = 4 * i as usize;
+		quadruple_word[i..i + 4].copy_from_slice(&word);
 	}
 
 	PciCapRaw {
@@ -1081,11 +1072,8 @@ fn read_caps(adapter: &PciAdapter, bars: Vec<PciBar>) -> Result<Vec<PciCap>, Pci
 						Some(bar) => {
 							// Drivers MUST ignore BAR values different then specified in Virtio spec v1.1. - 4.1.4
 							// See Virtio specification v1.1. - 4.1.4.1
-							if bar.index <= 5 {
-								if bar.index == cap_raw.bar_index {
-									// Need to clone here as every PciCap carrys it's bar
-									break bar.clone();
-								}
+							if bar.index <= 5 && bar.index == cap_raw.bar_index {
+								break *bar;
 							}
 						}
 						None => {
